namespace ReportLibrary
{
    using System;
    using Telerik.Reporting;
    using Telerik.Reporting.Expressions;

    public partial class SampleReport : Report
    {
        public SampleReport()
        {
            this.InitializeComponent();

            this.crosstabDataSource.DataSource = new[]
            {
                new SampleData { Subject = "Subject 1", Result = "Result 1", Value = 1 },
                new SampleData { Subject = "Subject 1", Result = "Result 2", Value = 1 },
                new SampleData { Subject = "Subject 1", Result = "Result 2", Value = 1 },
                new SampleData { Subject = "Subject 1", Result = "Result 3", Value = 1 },
                new SampleData { Subject = "Subject 1", Result = "Result 3", Value = 2 },
                new SampleData { Subject = "Subject 2", Result = "Result 1", Value = 1 },
                new SampleData { Subject = "Subject 2", Result = "Result 1", Value = 2 },
                new SampleData { Subject = "Subject 2", Result = "Result 1", Value = 3 },
                new SampleData { Subject = "Subject 2", Result = "Result 2", Value = 2 },
                new SampleData { Subject = "Subject 2", Result = "Result 2", Value = 3 },
                new SampleData { Subject = "Subject 2", Result = "Result 3", Value = 2 },
                new SampleData { Subject = "Subject 2", Result = "Result 3", Value = 2 },
                new SampleData { Subject = "Subject 3", Result = "Result 1", Value = 2 },
                new SampleData { Subject = "Subject 3", Result = "Result 1", Value = 3 },
                new SampleData { Subject = "Subject 3", Result = "Result 1", Value = 4 },
                new SampleData { Subject = "Subject 3", Result = "Result 2", Value = 3 },
                new SampleData { Subject = "Subject 3", Result = "Result 2", Value = 4 },
                new SampleData { Subject = "Subject 3", Result = "Result 3", Value = 1 },
                new SampleData { Subject = "Subject 3", Result = "Result 3", Value = 3 },
                new SampleData { Subject = "Subject 3", Result = "Result 3", Value = 4 }
            };

            var sortParameter = this.ReportParameters["SortBy"];

            sortParameter.AvailableValues.DataSource = new[]
            {
                "Subject 1",
                "Subject 2",
                "Subject 3"
            };
        }
    }

    public class SampleData
    {
        public string Subject { get; set; }
        public string Result { get; set; }
        public int Value { get; set; }
    }

    [AggregateFunction(Name = "SliceSum", Description = "Calculates the sum of values that match a certain criteria.")]
    public class SliceSum : IAggregateFunction
    {
        int totalValue;

        public void Init()
        {
            this.totalValue = 0;
        }

        public object GetValue()
        {
            return this.totalValue;
        }

        public void Accumulate(object[] values)
        {
            var filterValue = values[0];
            var slicedValue = values[1];

            if (object.Equals(filterValue, slicedValue))
            {
                var summedValue = values[2];

                if (null != summedValue)
                {
                    this.totalValue += Convert.ToInt32(summedValue);
                }
            }
        }

        public void Merge(IAggregateFunction aggregate)
        {
            var sliceSum = (SliceSum)aggregate;

            this.totalValue += sliceSum.totalValue;
        }
    }
}
